//------------------------------------------------------------------------
//
// OpenDCC - DCC Sniffer
//
// Copyright (c) 2007 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
// 
//------------------------------------------------------------------------
//
// file:      config.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2008-03-01 V2.00 kw start
//            2008-08-08 V2.03 kw added BiDi
//            2009-06-23 V2.04 kw added clock command
//         
//------------------------------------------------------------------------
//
// purpose:   flexible general purpose decoder for dcc
//            This is the central definition file for the project
//
//            Note: for Atmega162 and Bootloader: code limit is 14436
//------------------------------------------------------------------------
//
// content:   1. Project settings
//               1.a) General
//               1.b) Configuration of Software Modules
//
//            2. EEPROM definitions (CV's)
//            3. Global variables
//            4. Useful inline routines (delay, Semaphor-operations)
//
//========================================================================
//
#ifndef _CONFIG_H_
#define _CONFIG_H_

//========================================================================
// 1. Project Definitions
//========================================================================
//
// 1.a) General
//
#define SNIFFER_VERSION   0x02

#define SNIFFER_SERIAL    0x01     // This is a serial number used to identify different boxes.
#define BUILD_YEAR        0x08
#define BUILD_MONTH       0x05
#define BUILD_DAY         0x01

#define FALSE  0
#define TRUE   (!FALSE)

// Debugswitches:  0: no debugging, interrupts running -> alive!
//                 1: no ints, some lowlevel test

#define DEBUG  0



//========================================================================
// 2. EEPROM Definitions (CV's)
//========================================================================
//
// content is defined in config.c



//========================================================================
// 3. Global variables
//========================================================================

//---------------------------------------------------------------------
// Timing Definitions:
//

#define TICK_PERIOD 20000L       // 20ms tick for Timing Engine
                                 // => possible values for timings up to
                                 //    5.1s (=255/0.020)
                                 // note: this is also used as frame for
                                 // Servo-Outputs (OC1A and OC1B)

#define DEFAULT_BAUD      BAUD_115200  // supported: 2400, 4800, 9600, 19200, 38400, 57600, 115200

//----------------------------------------------------------------------------

extern volatile signed char timerval;    // gets incremented in the timetick



//========================================================================
// 4. Platform
//========================================================================
//
#define OPENSNIFFER    01

#define PLATFORM       OPENSNIFFER    

#define T0_PRESCALER   8    // may be 1, 8, 64, 256, 1024


#if defined(__AVR_ATmega168__)
#define DCC_ACK         PD0       // output, sending 1 makes an ACK
#define DCC_ACK_OFF     PORTD &= ~(1<<DCC_ACK)
#define DCC_ACK_ON      PORTD |= (1<<DCC_ACK)

#define DCC_BIT         PD2     // must be located on INT0
#define DCC_PORT        PORTD   // must be defined to have portable code
#define DCC_PORT_IN     PIND    // must be defined to have portable code

#elif defined (__AVR_ATtiny85__)
#define DCC_ACK         PB0       // output, sending 1 makes an ACK
#define DCC_ACK_OFF     PORTB &= ~(1<<DCC_ACK)
#define DCC_ACK_ON      PORTB |= (1<<DCC_ACK)

#define DCC_BIT         PB2     // must be located on INT0
#define DCC_PORT        PORTB   // must be defined to have portable code
#define DCC_PORT_IN     PINB    // must be defined to have portable code

#elif defined (__AVR_ATmega162__)
#define DCC_ACK         PB0       // output, sending 1 makes an ACK
#define DCC_ACK_OFF     PORTB &= ~(1<<DCC_ACK)
#define DCC_ACK_ON      PORTB |= (1<<DCC_ACK)

#define DCC_BIT         PD2     // must be located on INT0
#define DCC_PORT        PORTD   // must be defined to have portable code
#define DCC_PORT_IN     PIND    // must be defined to have portable code

#endif

//-------------------------------------------------------------------------
// 4.a) Definitions for inter process communication
//-------------------------------------------------------------------------
// These routines are *inline*, so we keep them in the header.
//
// usage:   semaphor_set(flag, state) to do the communication.
//          semaphor_get(flag)




extern volatile unsigned char Communicate;


#define C_Received        0    // a new DCC was received - issued by ISR(Timer1)
                               //                          cleared by main
#define C_DoSave          1    // a new PORT state should be saved
                               //                        - issued by action
                               //                          cleared by main
  

//========================================================================
// 4.b) Useful inline code
//========================================================================

static inline unsigned char semaphor_query(unsigned char flag) 
       __attribute__((always_inline));

unsigned char semaphor_query(unsigned char flag)
  {
    return (Communicate & (1<<flag));
  }
               
static inline void semaphor_set(unsigned char flag) 
       __attribute__((always_inline));

void semaphor_set(unsigned char flag)
  {
    cli();
    Communicate |= (1<<flag);
    sei();
  }
       
static inline unsigned char semaphor_get(unsigned char flag) 
       __attribute__((always_inline));

unsigned char semaphor_get(unsigned char flag)
  {
    unsigned char value;
    cli();
    value = Communicate & (1<<flag);
    Communicate &= (~(1<<flag));
    sei();
    return(value);
  }
        

//------------------------------------------------------------------------
// Delay-Macro (all values in us) -> this is busy waiting
//------------------------------------------------------------------------
//
// same macro as in util/delay.h, but use up to some ms.
// The maximal possible delay is 262.14 ms / F_CPU in MHz.
// This is 16ms for 16MHz; longest used delay: 1000us

#ifndef _UTIL_DELAY_H_
  #include <util/delay.h>
#endif

static inline void _mydelay_us(double __us) __attribute__((always_inline));
void
_mydelay_us(double __us)
{
    uint16_t __ticks;
    double __tmp = ((F_CPU) / 4e6) * __us;
    if (__tmp < 1.0)
        __ticks = 1;
    else if (__tmp > 65535)
        __ticks = 0;    /* i.e. 65536 */
    else
        __ticks = (uint16_t)__tmp;
    _delay_loop_2(__ticks);
}   

static inline void _restart(void) __attribute__((always_inline));
void
_restart(void)
{
    cli();
                    
    // laut diversen Internetseiten sollte folgender Code laufen -
    // tuts aber nicht, wenn man das Assemblerlistung ansieht.
    // void (*funcptr)( void ) = 0x0000;    // Set up function pointer
    // funcptr();                        // Jump to Reset vector 0x0000
    
    __asm__ __volatile 
    (
       "ldi r30,0"  "\n\t"
       "ldi r31,0"  "\n\t"
       "icall" "\n\t"
     );
}

#endif   // _config_h_
